@@ -13,16 +13,16 @@ subclass of PluginMainWidget.
 
 # Standard library imports
 from collections import OrderedDict
-import os
+import logging
 import sys
-import textwrap
 from typing import Optional
 
 # Third party imports
-from qtpy.QtCore import QSize, Qt, Signal, Slot
+from qtpy import PYQT5
+from qtpy.QtCore import QByteArray, QSize, Qt, Signal, Slot
 from qtpy.QtGui import QIcon
-from qtpy.QtWidgets import (QHBoxLayout, QSizePolicy, QToolButton, QVBoxLayout,
-                            QWidget)
+from qtpy.QtWidgets import (QApplication, QHBoxLayout, QSizePolicy,
+                            QToolButton, QVBoxLayout, QWidget)
 
 # Local imports
 from spyder.api.exceptions import SpyderAPIError
@@ -33,17 +33,21 @@ from spyder.api.widgets.menus import (MainWidgetMenu, OptionsMenuSections,
                                       PluginMainWidgetMenus)
 from spyder.api.widgets.mixins import SpyderToolbarMixin, SpyderWidgetMixin
 from spyder.api.widgets.toolbars import MainWidgetToolbar
+from spyder.py3compat import qbytearray_to_str
 from spyder.utils.qthelpers import create_waitspinner, set_menu_icons
 from spyder.utils.registries import (
     ACTION_REGISTRY, TOOLBAR_REGISTRY, MENU_REGISTRY)
 from spyder.utils.stylesheet import (
     APP_STYLESHEET, PANES_TABBAR_STYLESHEET, PANES_TOOLBAR_STYLESHEET)
-from spyder.widgets.dock import SpyderDockWidget
+from spyder.widgets.dock import DockTitleBar, SpyderDockWidget
 from spyder.widgets.tabs import Tabs
 
 # Localization
 _ = get_translation('spyder')
 
+# Logging
+logger = logging.getLogger(__name__)
+
 
 class PluginMainWidgetWidgets:
     CornerWidget = 'corner_widget'
@@ -56,6 +60,7 @@ class PluginMainWidgetActions:
     ClosePane = 'close_pane'
     DockPane = 'dock_pane'
     UndockPane = 'undock_pane'
+    LockUnlockPosition = 'lock_unlock_position'
 
 
 class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
@@ -188,7 +193,11 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
     """
 
     def __init__(self, name, plugin, parent=None):
-        super().__init__(parent=parent, class_parent=plugin)
+        if PYQT5:
+            super().__init__(parent=parent, class_parent=plugin)
+        else:
+            QWidget.__init__(self, parent)
+            SpyderWidgetMixin.__init__(self, class_parent=plugin)
 
         # Attributes
         # --------------------------------------------------------------------
@@ -228,6 +237,7 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
             parent=self,
             name=PluginMainWidgetWidgets.CornerWidget,
         )
+        self._corner_widget.ID = 'main_corner'
 
         self._main_toolbar = MainWidgetToolbar(
             parent=self,
@@ -329,6 +339,13 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
             icon=self.create_icon('dock'),
             triggered=self.close_window,
         )
+        self.lock_unlock_action = self.create_action(
+            name=PluginMainWidgetActions.LockUnlockPosition,
+            text=_("Unlock position"),
+            tip=_("Unlock to move pane to another position"),
+            icon=self.create_icon('drag_dock_widget'),
+            triggered=self.lock_unlock_position,
+        )
         self.undock_action = self.create_action(
             name=PluginMainWidgetActions.UndockPane,
             text=_("Undock"),
@@ -352,12 +369,12 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
             shortcut_context='_',
         )
 
-        bottom_section = OptionsMenuSections.Bottom
-        for item in [self.undock_action, self.close_action, self.dock_action]:
+        for item in [self.lock_unlock_action, self.undock_action,
+                     self.close_action, self.dock_action]:
             self.add_item_to_menu(
                 item,
                 self._options_menu,
-                section=bottom_section,
+                section=OptionsMenuSections.Bottom,
             )
 
         self._options_button.setMenu(self._options_menu)
@@ -380,21 +397,6 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
 
         # Update title
         self.setWindowTitle(self.get_title())
-        self._update_style()
-
-    def _update_style(self):
-        """
-        Update style of the widget.
-        """
-        qss = r"""
-            QToolButton::menu-indicator {
-                image: none;
-            }
-            QToolButton {
-                margin: 0px;
-            }
-            """
-        self._options_button.setStyleSheet(textwrap.dedent(qss))
 
     def _update_actions(self):
         """
@@ -403,6 +405,7 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         show_dock_actions = self.windowwidget is None
         self.undock_action.setVisible(show_dock_actions)
         self.close_action.setVisible(show_dock_actions)
+        self.lock_unlock_action.setVisible(show_dock_actions)
         self.dock_action.setVisible(not show_dock_actions)
 
         if sys.platform == 'darwin':
@@ -416,7 +419,6 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         # Widget setup
         # --------------------------------------------------------------------
         self.update_actions()
-        self._update_style()
 
     @Slot(bool)
     def _on_top_level_change(self, top_level):
@@ -425,6 +427,31 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         """
         self.undock_action.setDisabled(top_level)
 
+        # Change the cursor shape when dragging
+        if top_level:
+            QApplication.setOverrideCursor(Qt.ClosedHandCursor)
+        else:
+            QApplication.restoreOverrideCursor()
+
+    @Slot(bool)
+    def _on_title_bar_shown(self, visible):
+        """
+        Actions to perform when the title bar is shown/hidden.
+        """
+        if visible:
+            self.lock_unlock_action.setText(_('Lock position'))
+            self.lock_unlock_action.setIcon(self.create_icon('lock_open'))
+            for method_name in ['setToolTip', 'setStatusTip']:
+                method = getattr(self.lock_unlock_action, method_name)
+                method(_("Lock pane to the current position"))
+        else:
+            self.lock_unlock_action.setText(_('Unlock position'))
+            self.lock_unlock_action.setIcon(
+                self.create_icon('drag_dock_widget'))
+            for method_name in ['setToolTip', 'setStatusTip']:
+                method = getattr(self.lock_unlock_action, method_name)
+                method(_("Unlock to move pane to another position"))
+
     # --- Public Qt overriden methods
     # ------------------------------------------------------------------------
     def setLayout(self, layout):
@@ -436,6 +463,10 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         layout.setContentsMargins(self._margin_left, 0, self._margin_right, 0)
         layout.setSpacing(0)
 
+    def closeEvent(self, event):
+        self.on_close()
+        super().closeEvent(event)
+
     # --- Public methods to use
     # ------------------------------------------------------------------------
     def get_plugin(self):
@@ -555,11 +586,10 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
                 'Menu name "{}" already in use!'.format(menu_id)
             )
 
-        menu = MainWidgetMenu(parent=self, title=title)
-        menu.ID = menu_id
+        menu = MainWidgetMenu(parent=self, title=title, menu_id=menu_id)
 
         MENU_REGISTRY.register_reference(
-            menu, menu.ID, self.PLUGIN_NAME, self.CONTEXT_NAME)
+            menu, menu_id, self.PLUGIN_NAME, self.CONTEXT_NAME)
 
         if icon is not None:
             menu.menuAction().setIconVisibleInMenu(True)
@@ -710,8 +740,10 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
     @Slot()
     def create_window(self):
         """
-        Create a QMainWindow instance containing this SpyderWidget.
+        Create a QMainWindow instance containing this widget.
         """
+        logger.debug("Undocking plugin")
+
         # Widgets
         self.windowwidget = window = SpyderWindowWidget(self)
 
@@ -725,21 +757,51 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         window.setWindowTitle(self.get_title())
         window.resize(self.size())
 
+        # Restore window geometry
+        geometry = self.get_conf('window_geometry', default='')
+        if geometry:
+            try:
+                window.restoreGeometry(
+                    QByteArray().fromHex(str(geometry).encode('utf-8'))
+                )
+            except Exception:
+                pass
+
+        # Dock widget setup
         if self.dockwidget:
             self.dockwidget.setFloating(False)
             self.dockwidget.setVisible(False)
 
         self.set_ancestor(window)
         self._update_actions()
-        window.sig_closed.connect(self.close_window)
         window.show()
 
     @Slot()
-    def close_window(self):
+    def close_window(self, save_undocked=False):
         """
-        Close QMainWindow instance that contains this SpyderWidget.
+        Close QMainWindow instance that contains this widget.
+
+        Parameters
+        ----------
+        save_undocked : bool, optional
+            True if the undocked state needs to be saved. The default is False.
+
+        Returns
+        -------
+        None.
         """
+        logger.debug("Docking plugin back to the main window")
+
         if self.windowwidget is not None:
+            # Save window geometry to restore it when undocking the plugin
+            # again.
+            geometry = self.windowwidget.saveGeometry()
+            self.set_conf('window_geometry', qbytearray_to_str(geometry))
+
+            # Save undocking state if requested
+            if save_undocked:
+                self.set_conf('undocked_on_window_close', True)
+
             # Fixes spyder-ide/spyder#10704
             self.__unsafe_window = self.windowwidget
             self.__unsafe_window.deleteLater()
@@ -756,13 +818,12 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
                 self.dockwidget.setVisible(True)
                 self.dockwidget.raise_()
                 self._update_actions()
+        else:
+            # Reset undocked state
+            self.set_conf('undocked_on_window_close', False)
 
     def change_visibility(self, enable, force_focus=None):
-        """
-        Dock widget visibility has changed.
-        """
-        is_visible = not self.is_visible
-
+        """Dock widget visibility has changed."""
         if self.dockwidget is None:
             return
 
@@ -775,7 +836,7 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         raise_and_focus = getattr(self, 'RAISE_AND_FOCUS', None)
 
         if force_focus is None:
-            if raise_and_focus or not is_visible:
+            if raise_and_focus and enable:
                 focus_widget = self.get_focus_widget()
                 if focus_widget:
                     focus_widget.setFocus()
@@ -843,6 +904,7 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         # Signals
         dock.visibilityChanged.connect(self.change_visibility)
         dock.topLevelChanged.connect(self._on_top_level_change)
+        dock.sig_title_bar_shown.connect(self._on_title_bar_shown)
 
         return (dock, dock.LOCATION)
 
@@ -853,6 +915,15 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         """
         self.toggle_view(False)
 
+    def lock_unlock_position(self):
+        """
+        Show/hide title bar to move/lock position.
+        """
+        if isinstance(self.dockwidget.titleBarWidget(), DockTitleBar):
+            self.dockwidget.remove_title_bar()
+        else:
+            self.dockwidget.set_title_bar()
+
     # --- API: methods to define or override
     # ------------------------------------------------------------------------
     def get_title(self):
@@ -884,6 +955,13 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
             'A PluginMainWidget subclass must define an `update_actions` '
             f'method! Hint: {type(self)} should implement `update_actions`')
 
+    def on_close(self):
+        """
+        Perform actions before the widget is closed.
+
+        This method **must** only operate on local attributes.
+        """
+        pass
 
 def run_test():
     # Third party imports
